<?php
function generatePassword($length, $charTypes, $excludeChars, $uniqueChars) {
  $charSets = [
    'lowercase' => 'abcdefghijklmnopqrstuvwxyz',
    'uppercase' => 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
    'numbers' => '0123456789',
    'symbols' => '%&~!$()=#',
  ];

  foreach ($excludeChars as $excludeChar) {
    foreach ($charSets as &$charSet) {
      $charSet = str_replace($excludeChar, '', $charSet);
    }
  }

  $chars = '';
  foreach ($charTypes as $charType) {
    if (isset($charSets[$charType])) {
      $chars .= $charSets[$charType];
    }
  }

  if (empty($chars)) {
    return ['success' => false, 'message' => 'Please select at least one character type.'];
  }

  $password = '';
  $usedChars = '';
  for ($i = 0; $i < $length; $i++) {
    $randomIndex = mt_rand(0, strlen($chars) - 1);
    $randomChar = $chars[$randomIndex];

    if ($uniqueChars && strpos($usedChars, $randomChar) !== false) {
      $i--;
      continue;
    }

    $password .= $randomChar;
    $usedChars .= $randomChar;
  }

  $strengthPercentage = calculateStrengthPercentage($password);

  return [
    'success' => true,
    'password' => $password,
    'strengthPercentage' => $strengthPercentage,
    'message' => '',
  ];
}

function calculateStrengthPercentage($password) {
  $charTypesRegex = [
    '/[a-z]/', // Lowercase letters
    '/[A-Z]/', // Uppercase letters
    '/[0-9]/', // Numbers
    '/[%&~!$()=#]/', // Symbols
  ];

  $strengthPercentage = 0;
  foreach ($charTypesRegex as $regex) {
    if (preg_match($regex, $password)) {
      $strengthPercentage += 25;
    }
  }

  return $strengthPercentage;
}

$data = json_decode(file_get_contents('php://input'), true);

$length = isset($data['length']) ? (int)$data['length'] : 12;
$charTypes = isset($data['charTypes']) ? $data['charTypes'] : [];
$excludeChars = isset($data['excludeChars']) ? str_split($data['excludeChars']) : [];
$uniqueChars = isset($data['uniqueChars']) ? (bool)$data['uniqueChars'] : false;

$response = generatePassword($length, $charTypes, $excludeChars, $uniqueChars);

echo json_encode($response);
?>
